@@ -1,6 +1,6 @@
 // vim: set tabstop=4 shiftwidth=4 noexpandtab:
 /*	Gwenview - A simple image viewer for KDE
-	Copyright 2000-2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+	Copyright 2000-2007 Aurélien Gâteau <agateau@kde.org>
 	This class is based on the ImagePreviewJob class from Konqueror.
 */
 /*	This file is part of the KDE project
@@ -80,6 +80,69 @@ static QString generateThumbnailPath(const QString& uri, ThumbnailGroup::Enum gr
 	return baseDir + QString(QFile::encodeName( md5.hexDigest())) + ".png";
 }
 
+//------------------------------------------------------------------------
+//
+// ThumbnailCache
+//
+//------------------------------------------------------------------------
+K_GLOBAL_STATIC(ThumbnailCache, sThumbnailCache)
+
+
+static void storeThumbnailToDiskCache(const QString& path, const QImage& image) {
+	LOG(path);
+	KTemporaryFile tmp;
+	tmp.setPrefix(path + ".gwenview.tmp");
+	tmp.setSuffix(".png");
+	if (!tmp.open()) {
+		kWarning() << "Could not create a temporary file.";
+		return;
+	}
+
+	if (!image.save(tmp.fileName(), "png")) {
+		kWarning() << "Could not save thumbnail";
+		return;
+	}
+
+	KDE_rename(QFile::encodeName(tmp.fileName()), QFile::encodeName(path));
+}
+
+
+void ThumbnailCache::queueThumbnail(const QString& path, const QImage& image) {
+	LOG(path);
+	QMutexLocker locker(&mMutex);
+	mCache.insert(path, image);
+}
+
+void ThumbnailCache::run() {
+	QMutexLocker locker(&mMutex);
+	while (true) {
+		Cache::ConstIterator it = mCache.constBegin();
+		if (it == mCache.constEnd()) {
+			break;
+		}
+		const QString path = it.key();
+		const QImage image = it.value();
+
+		// This part of the thread is the most time consuming but it does not
+		// depend on mCache so we can unlock here. This way other thumbnails
+		// can be added or queried
+		locker.unlock();
+		storeThumbnailToDiskCache(path, image);
+		locker.relock();
+
+		mCache.remove(path);
+		if (mCache.isEmpty()) {
+			break;
+		}
+	}
+}
+
+QImage ThumbnailCache::value(const QString& path) const {
+	QMutexLocker locker(&mMutex);
+	return mCache.value(path);
+}
+
+
 //------------------------------------------------------------------------
 //
 // ThumbnailThread
@@ -173,13 +236,12 @@ void ThumbnailThread::loadThumbnail() {
 
 		if (qMax(thumbnail.width(), thumbnail.height()) >= pixelSize) {
 			mImage = thumbnail;
-			needCaching = false;
 		}
 	}
 
 	// Generate thumbnail from full image
 	if (mImage.isNull()) {
-		const QSize originalSize = reader.size();
+		QSize originalSize = reader.size();
 		if (originalSize.isValid() && reader.supportsOption(QImageIOHandler::ScaledSize)) {
 			int scale;
 			const int maxSize = qMax(originalSize.width(), originalSize.height());
@@ -191,13 +253,18 @@ void ThumbnailThread::loadThumbnail() {
 		}
 
 		QImage originalImage;
+		// format() is empty after QImageReader::read() is called
+		QByteArray format = reader.format();
 		if (reader.read(&originalImage)) {
+			if (!originalSize.isValid()) {
+				originalSize = originalImage.size();
+			}
 			mOriginalWidth = originalSize.width();
 			mOriginalHeight = originalSize.height();
 
-			if (qMax(mOriginalWidth, mOriginalHeight)<=pixelSize ) {
+			if (qMax(mOriginalWidth, mOriginalHeight)<=pixelSize) {
 				mImage=originalImage;
-				needCaching = false;
+				needCaching = format != "png";
 			} else {
 				mImage = originalImage.scaled(pixelSize, pixelSize, Qt::KeepAspectRatio);
 			}
@@ -216,40 +283,19 @@ void ThumbnailThread::loadThumbnail() {
 	}
 
 	if (needCaching) {
-		storeThumbnailInCache();
+		mImage.setText("Thumb::Uri"          , 0, mOriginalUri);
+		mImage.setText("Thumb::MTime"        , 0, QString::number(mOriginalTime));
+		mImage.setText("Thumb::Size"         , 0, QString::number(mOriginalSize));
+		mImage.setText("Thumb::Mimetype"     , 0, mOriginalMimeType);
+		mImage.setText("Thumb::Image::Width" , 0, QString::number(mOriginalWidth));
+		mImage.setText("Thumb::Image::Height", 0, QString::number(mOriginalHeight));
+		mImage.setText("Software"            , 0, "Gwenview");
+
+		emit thumbnailReadyToBeCached(mThumbnailPath, mImage);
 	}
 }
 
 
-void ThumbnailThread::storeThumbnailInCache() {
-	mImage.setText("Thumb::Uri"          , 0, mOriginalUri);
-	mImage.setText("Thumb::MTime"        , 0, QString::number(mOriginalTime));
-	mImage.setText("Thumb::Size"         , 0, QString::number(mOriginalSize));
-	mImage.setText("Thumb::Mimetype"     , 0, mOriginalMimeType);
-	mImage.setText("Thumb::Image::Width" , 0, QString::number(mOriginalWidth));
-	mImage.setText("Thumb::Image::Height", 0, QString::number(mOriginalHeight));
-	mImage.setText("Software"            , 0, "Gwenview");
-
-	QString thumbnailDir = ThumbnailLoadJob::thumbnailBaseDir(mThumbnailGroup);
-	KStandardDirs::makeDir(thumbnailDir, 0700);
-
-	KTemporaryFile tmp;
-	tmp.setPrefix(thumbnailDir + "/gwenview");
-	tmp.setSuffix(".png");
-	if (!tmp.open()) {
-		kWarning() << "Could not create a temporary file.";
-		return;
-	}
-
-	if (!mImage.save(tmp.fileName(), "png")) {
-		kWarning() << "Could not save thumbnail for file" << mOriginalUri;
-		return;
-	}
-
-	KDE_rename(QFile::encodeName(tmp.fileName()), QFile::encodeName(mThumbnailPath));
-}
-
-
 //------------------------------------------------------------------------
 //
 // ThumbnailLoadJob static methods
@@ -290,6 +336,26 @@ void ThumbnailLoadJob::deleteImageThumbnail(const KUrl& url) {
 }
 
 
+static void moveThumbnailHelper(const QString& oldUri, const QString& newUri, ThumbnailGroup::Enum group) {
+	QString oldPath = generateThumbnailPath(oldUri, group);
+	QString newPath = generateThumbnailPath(newUri, group);
+	QImage thumb;
+	if (!thumb.load(oldPath)) {
+		return;
+	}
+	thumb.setText("Thumb::Uri", 0, newUri);
+	thumb.save(newPath, "png");
+	QFile::remove(QFile::encodeName(oldPath));
+}
+
+void ThumbnailLoadJob::moveThumbnail(const KUrl& oldUrl, const KUrl& newUrl) {
+	QString oldUri = generateOriginalUri(oldUrl);
+	QString newUri = generateOriginalUri(newUrl);
+	moveThumbnailHelper(oldUri, newUri, ThumbnailGroup::Normal);
+	moveThumbnailHelper(oldUri, newUri, ThumbnailGroup::Large);
+}
+
+
 //------------------------------------------------------------------------
 //
 // ThumbnailLoadJob implementation
@@ -302,6 +368,10 @@ ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList& items, ThumbnailGroup::E
 {
 	LOG((int)this);
 
+	// Make sure we have a place to store our thumbnails
+	QString thumbnailDir = ThumbnailLoadJob::thumbnailBaseDir(mThumbnailGroup);
+	KStandardDirs::makeDir(thumbnailDir, 0700);
+
 	// Look for images and store the items in our todo list
 	Q_ASSERT(!items.empty());
 	mItems = items;
@@ -310,6 +380,10 @@ ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList& items, ThumbnailGroup::E
 	connect(&mThumbnailThread, SIGNAL(done(const QImage&, const QSize&)),
 		SLOT(thumbnailReady(const QImage&, const QSize&)),
 		Qt::QueuedConnection);
+
+	connect(&mThumbnailThread, SIGNAL(thumbnailReadyToBeCached(const QString&, const QImage&)),
+		sThumbnailCache, SLOT(queueThumbnail(const QString&, const QImage&)),
+		Qt::QueuedConnection);
 }
 
 
@@ -323,6 +397,9 @@ ThumbnailLoadJob::~ThumbnailLoadJob() {
 	}
 	mThumbnailThread.cancel();
 	mThumbnailThread.wait();
+	if (!sThumbnailCache->isRunning()) {
+		sThumbnailCache->start();
+	}
 }
 
 
@@ -483,12 +560,20 @@ void ThumbnailLoadJob::thumbnailReady( const QImage& _img, const QSize& _size) {
 	determineNextIcon();
 }
 
+QImage ThumbnailLoadJob::loadThumbnailFromCache() const {
+	QImage image = sThumbnailCache->value(mThumbnailPath);
+	if (!image.isNull()) {
+		return image;
+	}
+	return QImage(mThumbnailPath);
+}
+
 void ThumbnailLoadJob::checkThumbnail() {
 	// If we are in the thumbnail dir, just load the file
 	if (mCurrentUrl.isLocalFile()
 		&& mCurrentUrl.directory().startsWith(thumbnailBaseDir()) )
 	{
-		QImage image(mCurrentUrl.path());
+		QImage image(mCurrentUrl.toLocalFile());
 		emitThumbnailLoaded(image, image.size());
 		determineNextIcon();
 		return;
@@ -500,8 +585,8 @@ void ThumbnailLoadJob::checkThumbnail() {
 
 	LOG("Stat thumb" << mThumbnailPath);
 
-	QImage thumb;
-	if ( thumb.load(mThumbnailPath) ) {
+	QImage thumb = loadThumbnailFromCache();
+	if (!thumb.isNull()) {
 		if (thumb.text("Thumb::Uri", 0) == mOriginalUri &&
 			thumb.text("Thumb::MTime", 0).toInt() == mOriginalTime )
 		{
@@ -534,11 +619,10 @@ void ThumbnailLoadJob::checkThumbnail() {
 	}
 
 	// Thumbnail not found or not valid
-	if ( MimeTypeUtils::rasterImageMimeTypes().contains(mCurrentItem.mimetype()) ) {
-		// This is a raster image
+	if (MimeTypeUtils::fileItemKind(mCurrentItem) == MimeTypeUtils::KIND_RASTER_IMAGE) {
 		if (mCurrentUrl.isLocalFile()) {
 			// Original is a local file, create the thumbnail
-			startCreatingThumbnail(mCurrentUrl.path());
+			startCreatingThumbnail(mCurrentUrl.toLocalFile());
 		} else {
 			// Original is remote, download it
 			mState=STATE_DOWNLOADORIG;
