@@ -6,8 +6,8 @@
  * Date        : 2006-09-15
  * Description : Exiv2 library interface for KDE
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * Exiv2: http://www.exiv2.org
  * Exif : http://www.exif.org/Exif2-2.PDF
@@ -38,12 +38,14 @@
 #include <QtGui/QImage>
 #include <QtCore/QDateTime>
 #include <QtCore/QMap>
+#include <QtCore/QSharedDataPointer>
 #include <QtCore/QStringList>
 #include <QtCore/QVariant>
 
 // Local includes.
 
 #include "libkexiv2_export.h"
+#include "kexiv2data.h"
 
 namespace KExiv2Iface
 {
@@ -80,8 +82,7 @@ public:
         ORIENTATION_ROT_270      = 8
     };
 
-    /** A map used by decodeExifMetadata() decodeIptcMetadata() methods
-        to store Tags Key and Tags Value. 
+    /** A map used to store Tags Key and Tags Value.
      */
     typedef QMap<QString, QString> MetaDataMap;
 
@@ -91,6 +92,13 @@ public:
      */
     typedef QMap<QString, QString> AltLangMap; 
 
+    /** A map used to store Tags Key and a list of Tags properties :
+        - name,
+        - title,
+        - description.
+     */
+    typedef QMap<QString, QStringList> TagsMap;
+
 public:
 
     /** Standard constructor.
@@ -101,6 +109,10 @@ public:
      */
     KExiv2(const KExiv2& metadata);
 
+    /** Constructor to load from parsed data
+     */
+    KExiv2(const KExiv2Data& data);
+
     /** Contructor to Load Metadata from image file.
      */
     KExiv2(const QString& filePath);
@@ -109,7 +121,7 @@ public:
      */
     virtual ~KExiv2();
 
-    /** Create a deep copy of container
+    /** Create a copy of container
      */
     KExiv2& operator=(const KExiv2& metadata);
 
@@ -149,20 +161,18 @@ public:
      */
     static QString version();
 
-    /** return true if metadata from file cannot be written by Exiv2.
-        This method is obosolete and will be removed.
-        Use canWriteComment(), canWriteExif(), canWriteIptc(), or canWriteXmp() instead.
-     */
-    KDE_DEPRECATED static bool isReadOnly(const QString& filePath);
-
     //-----------------------------------------------------------------
     //-- GENERAL methods ----------------------------------------------
     //-----------------------------------------------------------------
 
+    KExiv2Data data() const;
+    void setData(const KExiv2Data& data);
+
     /** Load all metadata (Exif, Iptc, Xmp, and JFIF Comments) from a byte array. 
         Return true if metadata have been loaded successfully from image data.
      */
-    bool load(const QByteArray& imgData) const;
+    bool loadFromData(const QByteArray& imgData) const;
+    KDE_DEPRECATED bool load(const QByteArray& imgData) const;
 
     /** Load all metadata (Exif, Iptc, Xmp, and JFIF Comments) from a picture (JPEG, RAW, TIFF, PNG,
         DNG, etc...). Return true if metadata have been loaded successfully from file.
@@ -201,6 +211,15 @@ public:
      */
     bool writeRawFiles() const;
 
+    /** Enable or disable file timestamp updating when metadata are saved.
+        By default files timestamp are untouched.
+     */
+    void setUpdateFileTimeStamp(bool on);
+
+    /** Return true if file timestamp is updated when metadata are saved.
+     */
+    bool updateFileTimeStamp() const;
+
     //-------------------------------------------------------------------
     //-- Metadata IMAGE INFORMATION manipulation methods ----------------
     //-------------------------------------------------------------------
@@ -313,6 +332,14 @@ public:
     //-- EXIF manipulation methods ------------------------------------
     //-----------------------------------------------------------------
 
+    /** Return a map of all standard Exif tags supported by Exiv2.
+     */
+    TagsMap getStdExifTagsList() const;
+
+    /** Return a map of all non-standard Exif tags (makernotes) supported by Exiv2.
+     */
+    TagsMap getMakernoteTagsList() const;
+
     /** Return 'true' if Exif can be written in file.
      */
     static bool canWriteExif(const QString& filePath);
@@ -325,11 +352,15 @@ public:
      */
     bool clearExif() const;
 
-    /** Return a Qt byte array copy of Exif container get from current image. 
-        Set true 'addExifHeader' parameter to add an Exif header to Exif metadata. 
-        Return a null Qt byte array if there is no Exif metadata in memory.
+    /** Returns the exif data encoded to a QByteArray in a form suitable
+        for storage in a JPEG image.
+        Note that this encoding is a lossy operation.
+
+        Set true 'addExifHeader' parameter to add an Exif header to Exif metadata.
+        Returns a null Qt byte array if there is no Exif metadata in memory.
      */
-    QByteArray getExif(bool addExifHeader=false) const;
+    KDE_DEPRECATED QByteArray getExif(bool addExifHeader=false) const;
+    QByteArray getExifEncoded(bool addExifHeader=false) const;
 
     /** Set the Exif data using a Qt byte array. Return true if Exif metadata
         have been changed in memory.
@@ -463,6 +494,10 @@ public:
     //-- IPTC manipulation methods --------------------------------
     //-------------------------------------------------------------
 
+    /** Return a map of all standard Iptc tags supported by Exiv2.
+     */
+    KExiv2::TagsMap getIptcTagsList() const;
+
     /** Return 'true' if Iptc can be written in file.
      */
     static bool canWriteIptc(const QString& filePath);
@@ -592,6 +627,10 @@ public:
     //-- XMP manipulation methods --------------------------------
     //------------------------------------------------------------
 
+    /** Return a map of all standard Xmp tags supported by Exiv2.
+     */
+    KExiv2::TagsMap getXmpTagsList() const;
+
     /** Return 'true' if Xmp can be written in file.
      */
     static bool canWriteXmp(const QString& filePath);
@@ -700,6 +739,21 @@ public:
     bool setXmpTagStringBag(const char *xmpTagName, const QStringList& bag,
                             bool setProgramName=true) const;
 
+    /** Set an Xmp tag content using a list of strings defined by the 'entriesToAdd' parameter.
+        The existing entries are preserved. The method will compare
+        all new with all already existing entries to prevent duplicates in the image.
+        Return true if the entries have been added to metadata.
+     */
+    bool addToXmpTagStringBag(const char *xmpTagName, const QStringList& entriesToAdd,
+                              bool setProgramName) const;
+
+    /** Remove those Xmp tag entries that are listed in entriesToRemove from the entries in metadata.
+        Return true if tag entries are no longer contained in metadata.
+        All other entries are preserved.
+     */
+    bool removeFromXmpTagStringBag(const char *xmpTagName, const QStringList& entriesToRemove,
+                                   bool setProgramName) const;
+
     /** Get an Xmp tag content as a QVariant. Returns a null QVariant if the Xmp
         tag cannot be found.
         For string and integer values the matching QVariant types will be used,
@@ -723,6 +777,11 @@ public:
      */
     bool setXmpKeywords(const QStringList& newKeywords, bool setProgramName=true) const;
 
+    /** Remove those Xmp keywords that are listed in keywordsToRemove from the keywords in metadata.
+        Return true if keywords are no longer contained in metadata.
+     */
+    bool removeXmpKeywords(const QStringList& keywordsToRemove, bool setProgramName=true);
+
     /** Return a strings list of Xmp subjects from image. Return an empty list if no subject are set.
      */
     QStringList getXmpSubjects() const;
@@ -734,6 +793,11 @@ public:
      */
     bool setXmpSubjects(const QStringList& newSubjects, bool setProgramName=true) const;
 
+    /** Remove those Xmp subjects that are listed in subjectsToRemove from the subjects in metadata.
+        Return true if subjects are no longer contained in metadata.
+     */
+    bool removeXmpSubjects(const QStringList& subjectsToRemove, bool setProgramName=true);
+
     /** Return a strings list of Xmp sub-categories from image. Return an empty list if no sub-category 
         are set.
      */
@@ -746,18 +810,27 @@ public:
      */
     bool setXmpSubCategories(const QStringList& newSubCategories, bool setProgramName=true) const;
 
+    /** Remove those Xmp sub-categories that are listed in categoriesToRemove from the sub-categories in metadata.
+        Return true if subjects are no longer contained in metadata.
+     */
+    bool removeXmpSubCategories(const QStringList& categoriesToRemove, bool setProgramName=true);
+
+    /** Remove the Xmp tag 'xmpTagName' from Xmp metadata. Return true if tag is
+        removed successfully or if no tag was present.
+     */
+    bool removeXmpTag(const char *xmpTagName, bool setProgramName=true) const;
+
+
     /** Register a namespace which Exiv2 doesn't know yet. This is only needed
         when new Xmp properties are added manually. 'uri' is the namespace url and prefix the 
         string used to construct new Xmp key (ex. "Xmp.digiKam.tagList").
         NOTE: If the Xmp metadata is read from an image, namespaces are decoded and registered 
         by Exiv2 at the same time.
      */
-    bool registerXmpNameSpace(const QString& uri, const QString& prefix) const;
+    static bool registerXmpNameSpace(const QString& uri, const QString& prefix);
 
-    /** Remove the Xmp tag 'xmpTagName' from Xmp metadata. Return true if tag is 
-        removed successfully or if no tag was present.
-     */
-    bool removeXmpTag(const char *xmpTagName, bool setProgramName=true) const;
+    /** Unregister a previously registered custom namespace */
+    static bool unregisterXmpNameSpace(const QString& uri);
 
     //------------------------------------------------------------
     //-- GPS manipulation methods --------------------------------
@@ -882,7 +955,7 @@ private:
 
     /** Internal class to store private members. Used to improve binary compatibility
      */
-    KExiv2Priv *d;
+    KExiv2Priv* const d;
 };
 
 }  // NameSpace KExiv2Iface
