@@ -18,6 +18,7 @@
 #include <qlayout.h>
 #include <qmutex.h>
 #include <qregexp.h>
+#include <qstack.h>
 #include <qtextstream.h>
 #include <QtGui/QPrinter>
 
@@ -170,9 +171,25 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 		break;
 	
 		case Poppler::Link::Goto:
+		{
 			popplerLinkGoto = static_cast<const Poppler::LinkGoto *>(popplerLink);
+#ifdef HAVE_POPPLER_0_11
+			const Poppler::LinkDestination dest = popplerLinkGoto->destination();
+			const QString destName = dest.destinationName();
+			if (destName.isEmpty())
+			{
+				fillViewportFromLinkDestination( viewport, dest );
+				link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+			}
+			else
+			{
+				link = new Okular::GotoAction(popplerLinkGoto->fileName(), destName);
+			}
+#else
 			fillViewportFromLinkDestination( viewport, popplerLinkGoto->destination() );
 			link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+#endif
+		}
 		break;
 		
 		case Poppler::Link::Execute:
@@ -258,7 +275,7 @@ static KAboutData createAboutData()
          "okular_poppler",
          "okular_poppler",
          ki18n( "PDF Backend" ),
-         "0.2",
+         "0.3",
          ki18n( "A PDF file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2008 Albert Astals Cid" )
@@ -272,7 +289,9 @@ OKULAR_EXPORT_PLUGIN(PDFGenerator, createAboutData())
 PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     : Generator( parent, args ), pdfdoc( 0 ), ready( true ),
     pixmapRequest( 0 ), docInfoDirty( true ), docSynopsisDirty( true ),
-    docEmbeddedFilesDirty( true ), nextFontPage( 0 )
+    docEmbeddedFilesDirty( true ), nextFontPage( 0 ),
+    dpiX( 72.0 /*Okular::Utils::dpiX()*/ ), dpiY( 72.0 /*Okular::Utils::dpiY()*/ ),
+    synctex_scanner(0)
 {
     setFeature( TextExtraction );
     setFeature( FontInfo );
@@ -310,9 +329,15 @@ bool PDFGenerator::loadDocument( const QString & filePath, QVector<Okular::Page*
     // create PDFDoc for the given file
     pdfdoc = Poppler::Document::load( filePath, 0, 0 );
     bool success = init(pagesVector, filePath.section('/', -1, -1));
-    if (success && QFile::exists(filePath + QLatin1String( "sync" )))
+    if (success)
     {
-        loadPdfSync(filePath, pagesVector);
+        // no need to check for the existence of a synctex file, no parser will be
+        // created if none exists
+        initSynctexParser(filePath);
+        if ( !synctex_scanner && QFile::exists(filePath + QLatin1String( "sync" ) ) )
+        {
+            loadPdfSync(filePath, pagesVector);
+        }
     }
     return success;
 }
@@ -430,6 +455,12 @@ bool PDFGenerator::doCloseDocument()
     qDeleteAll(docEmbeddedFiles);
     docEmbeddedFiles.clear();
     nextFontPage = 0;
+    rectsGenerated.clear();
+    if ( synctex_scanner )
+    {
+        synctex_scanner_free( synctex_scanner );
+        synctex_scanner = 0;
+    }
 
     return true;
 }
@@ -437,14 +468,15 @@ bool PDFGenerator::doCloseDocument()
 void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation, bool clear)
 {
     // TODO XPDF 3.01 check
-    int count=pagesVector.count(),w=0,h=0;
+    const int count = pagesVector.count();
+    double w = 0, h = 0;
     for ( int i = 0; i < count ; i++ )
     {
         // get xpdf page
         Poppler::Page * p = pdfdoc->page( i );
-        QSize pSize = p->pageSize();
-        w = pSize.width();
-        h = pSize.height();
+        const QSizeF pSize = p->pageSizeF();
+        w = pSize.width() / 72.0 * dpiX;
+        h = pSize.height() / 72.0 * dpiY;
         Okular::Rotation orientation = Okular::Rotation0;
         switch (p->orientation())
         {
@@ -513,7 +545,7 @@ const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
                          KGlobal::locale()->formatDateTime( pdfdoc->date("ModDate"), KLocale::LongDate, true ) );
 
             docInfo.set( "format", i18nc( "PDF v. <version>", "PDF v. %1",
-                          pdfdoc->pdfVersion() ), i18n( "Format" ) );
+                         QString::number( pdfdoc->pdfVersion() ) ), i18n( "Format" ) );
             docInfo.set( "encryption", pdfdoc->isEncrypted() ? i18n( "Encrypted" ) : i18n( "Unencrypted" ),
                          i18n("Security") );
             docInfo.set( "optimization", pdfdoc->isLinearized() ? i18n( "Yes" ) : i18n( "No" ),
@@ -743,8 +775,8 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     if ( page->rotation() % 2 )
         qSwap( pageWidth, pageHeight );
 
-    double fakeDpiX = request->width() * 72.0 / pageWidth,
-           fakeDpiY = request->height() * 72.0 / pageHeight;
+    double fakeDpiX = request->width() * dpiX / pageWidth,
+           fakeDpiY = request->height() * dpiY / pageHeight;
 
     // setup Okular:: output device: text page is generated only if we are at 72dpi.
     // since we can pre-generate the TextPage at the right res.. why not?
@@ -782,12 +814,9 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     userMutex()->unlock();
     if ( genTextPage )
     {
-#ifdef HAVE_POPPLER_0_7
         QList<Poppler::TextBox*> textList = p->textList();
-#else
-        QList<Poppler::TextBox*> textList = p->textList((Poppler::Page::Rotation)request->page()->orientation());
-#endif
-        Okular::TextPage *tp = abstractTextPage(textList, page->height(), page->width(), request->page()->orientation());
+        const QSizeF s = p->pageSizeF();
+        Okular::TextPage *tp = abstractTextPage(textList, s.height(), s.width(), request->page()->orientation());
         page->setTextPage( tp );
         qDeleteAll(textList);
         
@@ -811,21 +840,14 @@ Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
     // build a TextList...
     Poppler::Page *pp = pdfdoc->page( page->number() );
     userMutex()->lock();
-#ifdef HAVE_POPPLER_0_7
     QList<Poppler::TextBox*> textList = pp->textList();
-#else
-    QList<Poppler::TextBox*> textList = pp->textList((Poppler::Page::Rotation)page->orientation());
-#endif
     userMutex()->unlock();
-    delete pp;
 
-#ifdef HAVE_POPPLER_0_7
-    const double pageWidth = page->width();
-    const double pageHeight = page->height();
-#else
-    const double pageWidth = ( page->rotation() % 2 ? page->height() : page->width() );
-    const double pageHeight = ( page->rotation() % 2 ? page->width() : page->height() );
-#endif
+    QSizeF s = pp->pageSizeF();
+    const double pageWidth = s.width();
+    const double pageHeight = s.height();
+
+    delete pp;
 
     Okular::TextPage *tp = abstractTextPage(textList, pageHeight, pageWidth, (Poppler::Page::Rotation)page->orientation());
     qDeleteAll(textList);
@@ -843,8 +865,10 @@ void PDFGenerator::requestFontData(const Okular::FontInfo &font, QByteArray *dat
 #endif
 }
 
+#define DUMMY_QPRINTER_COPY
 bool PDFGenerator::print( QPrinter& printer )
 {
+#ifdef DUMMY_QPRINTER_COPY
     // Get the real page size to pass to the ps generator
     QPrinter dummy( QPrinter::PrinterResolution );
     dummy.setFullPage( true );
@@ -853,6 +877,10 @@ bool PDFGenerator::print( QPrinter& printer )
     dummy.setPaperSize( printer.paperSize( QPrinter::Millimeter ), QPrinter::Millimeter );
     int width = dummy.width();
     int height = dummy.height();
+#else
+    int width = printer.width();
+    int height = printer.height();
+#endif
 
     // Create the tempfile to send to FilePrinter, which will manage the deletion
     KTemporaryFile tf;
@@ -867,11 +895,7 @@ bool PDFGenerator::print( QPrinter& printer )
 
     // TODO rotation
 
-#ifdef HAVE_POPPLER_0_7
     tf.setAutoRemove(false);
-#else
-    tf.close();
-#endif
 
     QString pstitle = metaData(QLatin1String("Title"), QVariant()).toString();
     if ( pstitle.trimmed().isEmpty() )
@@ -887,11 +911,7 @@ bool PDFGenerator::print( QPrinter& printer )
 
     Poppler::PSConverter *psConverter = pdfdoc->psConverter();
 
-#ifdef HAVE_POPPLER_0_7
     psConverter->setOutputDevice(&tf);
-#else
-    psConverter->setOutputFileName(tempfilename);
-#endif
 
     psConverter->setPageList(pageList);
     psConverter->setPaperWidth(width);
@@ -909,9 +929,7 @@ bool PDFGenerator::print( QPrinter& printer )
     {
         userMutex()->unlock();
         delete psConverter;
-#ifdef HAVE_POPPLER_0_7
         tf.close();
-#endif
         int ret = Okular::FilePrinter::printFile( printer, tempfilename,
                                                   Okular::FilePrinter::SystemDeletesFiles,
                                                   Okular::FilePrinter::ApplicationSelectsPages,
@@ -940,17 +958,28 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
     }
     else if ( key == "NamedViewport" && !option.toString().isEmpty() )
     {
-        // asking for the page related to a 'named link destination'. the
-        // option is the link name. @see addSynopsisChildren.
         Okular::DocumentViewport viewport;
-        userMutex()->lock();
-        Poppler::LinkDestination *ld = pdfdoc->linkDestination( option.toString() );
-        userMutex()->unlock();
-        if ( ld )
+        QString optionString = option.toString();
+
+        // if option starts with "src:" assume that we are handling a 
+        // source reference
+        if ( optionString.startsWith( "src:", Qt::CaseInsensitive ) )
         {
-            fillViewportFromLinkDestination( viewport, *ld );
+            fillViewportFromSourceReference( viewport, optionString );
+        }
+        else
+        {
+            // asking for the page related to a 'named link destination'. the
+            // option is the link name. @see addSynopsisChildren.
+            userMutex()->lock();
+            Poppler::LinkDestination *ld = pdfdoc->linkDestination( optionString );
+            userMutex()->unlock();
+            if ( ld )
+            {
+                fillViewportFromLinkDestination( viewport, *ld );
+            }
+            delete ld;
         }
-        delete ld;
         if ( viewport.pageNumber >= 0 )
             return viewport.toString();
     }
@@ -993,7 +1022,7 @@ bool PDFGenerator::reparseConfig()
         userMutex()->unlock();
         somethingchanged = true;
     }
-    bool aaChanged = setAAOptions();
+    bool aaChanged = setDocumentRenderHints();
     somethingchanged = somethingchanged || aaChanged;
     return somethingchanged;
 }
@@ -1002,7 +1031,7 @@ void PDFGenerator::addPages( KConfigDialog * )
 {
 }
 
-bool PDFGenerator::setAAOptions()
+bool PDFGenerator::setDocumentRenderHints()
 {
     bool changed = false;
     static Poppler::Document::RenderHints oldhints = 0;
@@ -1021,6 +1050,9 @@ bool PDFGenerator::setAAOptions()
 }
     SET_HINT("GraphicsAntialias", true, Poppler::Document::Antialiasing)
     SET_HINT("TextAntialias", true, Poppler::Document::TextAntialiasing)
+#ifdef HAVE_POPPLER_0_12_1
+    SET_HINT("TextHinting", false, Poppler::Document::TextHinting)
+#endif
 #undef SET_HINT
     return changed;
 }
@@ -1086,7 +1118,6 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
     int charCount=0;
     int j;
     QString s;
-#ifdef HAVE_POPPLER_0_7
     foreach (Poppler::TextBox *word, text)
     {
         charCount=word->text().length();
@@ -1117,117 +1148,6 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
                      wordBBox.top()/height);
         }
     }
-#else
-    Okular::NormalizedRect * wordRect = new Okular::NormalizedRect;
-    
-    rot = rot % 4;
-    
-    foreach (Poppler::TextBox *word, text)
-    {
-        wordRect->left = word->boundingBox().left();
-        wordRect->bottom = word->boundingBox().bottom();
-        wordRect->right = word->boundingBox().right();
-        wordRect->top = word->boundingBox().top();
-        charCount=word->text().length();
-        next=word->nextWord();
-        switch (rot)
-        {
-            case 0:
-            // 0 degrees, normal word boundaries are top and bottom
-            // only x boundaries change the order of letters is normal not reversed
-            for (j = 0; j < charCount; j++)
-            {
-                s = word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    // this letters boundary
-                    word->edge(j)/width,
-                    wordRect->bottom/height,
-                    // next letters boundary
-                    word->edge(j+1)/width,
-                    wordRect->top/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     word->edge(charCount)/width,
-                     wordRect->bottom/height,
-                    // next letters boundary
-                     next->edge(0)/width,
-                     wordRect->top/height);
-            break;
-
-            case 1:
-            // 90 degrees, x boundaries not changed
-            // y ones change, the order of letters is normal not reversed
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    wordRect->left/width,
-                    word->edge(j)/height,
-                    wordRect->right/width,
-                    word->edge(j+1)/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     wordRect->left/width,
-                     word->edge(charCount)/height,
-                    // next letters boundary
-                     wordRect->right/width,
-                     next->edge(0)/height);
-            break;
-
-            case 2:
-            // same as case 0 but reversed order of letters
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    word->edge(j+1)/width,
-                    wordRect->bottom/height,
-                    word->edge(j)/width,
-                    wordRect->top/height);
-
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     next->edge(0)/width,
-                     wordRect->bottom/height,
-                    // next letters boundary
-                     word->edge(charCount)/width,
-                     wordRect->top/height);
-           
-            break;
-
-            case 3:
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    wordRect->left/width,
-                    word->edge(j+1)/height,
-                    wordRect->right/width,
-                    word->edge(j)/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     wordRect->left/width,
-                     next->edge(0)/height,
-                    // next letters boundary
-                     wordRect->right/width,
-                     word->edge(charCount)/height);
-            break;
-        }
-    }
-    delete wordRect;
-#endif
     return ktp;
 }
 
@@ -1400,11 +1320,9 @@ void PDFGenerator::addFormFields( Poppler::Page * popplerPage, Okular::Page * pa
         Okular::FormField * of = 0;
         switch ( f->type() )
         {
-#ifdef HAVE_POPPLER_0_7
             case Poppler::FormField::FormButton:
                 of = new PopplerFormFieldButton( static_cast<Poppler::FormFieldButton*>( f ) );
                 break;
-#endif
             case Poppler::FormField::FormText:
                 of = new PopplerFormFieldText( static_cast<Poppler::FormFieldText*>( f ) );
                 break;
@@ -1451,72 +1369,76 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         return;
 
     QHash<int, pdfsyncpoint> points;
-    QString currentfile;
+    QStack<QString> fileStack;
     int currentpage = -1;
-    QRegExp newfilere( "\\(\\s*([^\\s]+)" );
-    QRegExp linere( "l\\s+(\\d+)\\s+(\\d+)(\\s+(\\d+))?" );
-    QRegExp pagere( "s\\s+(\\d+)" );
-    QRegExp locre( "p\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
-    QRegExp locstarre( "p\\*\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
+    const QLatin1String texStr( ".tex" );
+    const QChar spaceChar = QChar::fromLatin1( ' ' );
+
+    fileStack.push( coreName + texStr );
 
     QString line;
     while ( !ts.atEnd() )
     {
         line = ts.readLine();
-        if ( line.startsWith( QLatin1Char( 'l' ) ) && linere.exactMatch( line ) )
+        const QStringList tokens = line.split( spaceChar, QString::SkipEmptyParts );
+        const int tokenSize = tokens.count();
+        if ( tokenSize < 1 )
+            continue;
+        if ( tokens.first() == QLatin1String( "l" ) && tokenSize >= 3 )
         {
-            int id = linere.cap( 1 ).toInt();
+            int id = tokens.at( 1 ).toInt();
             QHash<int, pdfsyncpoint>::const_iterator it = points.constFind( id );
             if ( it == points.constEnd() )
             {
                 pdfsyncpoint pt;
                 pt.x = 0;
                 pt.y = 0;
-                pt.row = linere.cap( 2 ).toInt();
+                pt.row = tokens.at( 2 ).toInt();
                 pt.column = 0; // TODO
                 pt.page = -1;
-                pt.file = currentfile;
+                pt.file = fileStack.top();
                 points[ id ] = pt;
             }
         }
-        else if ( line.startsWith( QLatin1Char( 's' ) ) && pagere.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "s" ) && tokenSize >= 2 )
         {
-            currentpage = pagere.cap( 1 ).toInt() - 1;
+            currentpage = tokens.at( 1 ).toInt() - 1;
         }
-        else if ( line.startsWith( QLatin1String( "p*" ) ) && locstarre.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "p*" ) && tokenSize >= 4 )
         {
             // TODO
             kDebug(PDFDebug) << "PdfSync: 'p*' line ignored";
         }
-        else if ( line.startsWith( QLatin1Char( 'p' ) ) && locre.exactMatch( line ) )
+        else if ( tokens.first() == QLatin1String( "p" ) && tokenSize >= 4 )
         {
-            int id = locre.cap( 1 ).toInt();
+            int id = tokens.at( 1 ).toInt();
             QHash<int, pdfsyncpoint>::iterator it = points.find( id );
             if ( it != points.end() )
             {
-                it->x = locre.cap( 2 ).toInt();
-                it->y = locre.cap( 3 ).toInt();
+                it->x = tokens.at( 2 ).toInt();
+                it->y = tokens.at( 3 ).toInt();
                 it->page = currentpage;
             }
         }
-        else if ( line.startsWith( QLatin1Char( '(' ) ) && newfilere.exactMatch( line ) )
+        else if ( line.startsWith( QLatin1Char( '(' ) ) && tokenSize == 1 )
         {
-            QString newfile = newfilere.cap( 1 );
-            if ( currentfile.isEmpty() )
+            QString newfile = line;
+            // chop the leading '('
+            newfile.remove( 0, 1 );
+            if ( !newfile.endsWith( texStr ) )
             {
-                currentfile = newfile;
+                newfile += texStr;
             }
-            else
-                kDebug(PDFDebug) << "PdfSync: more than one file level:" << newfile;
+            fileStack.push( newfile );
         }
         else if ( line == QLatin1String( ")" ) )
         {
-            if ( !currentfile.isEmpty() )
+            if ( !fileStack.isEmpty() )
             {
-                currentfile.clear();
+                fileStack.pop();
             }
             else
-                kDebug(PDFDebug) << "PdfSync: going one level down:" << currentfile;
+                kDebug(PDFDebug) << "PdfSync: going one level down too much";
         }
         else
             kDebug(PDFDebug).nospace() << "PdfSync: unknown line format: '" << line << "'";
@@ -1530,24 +1452,12 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         if ( pt.page < 0 || pt.page >= pagesVector.size() )
             continue;
 
-        // maginc numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
+        // magic numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
         Okular::NormalizedPoint p(
-            ( pt.x * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
-            ( pt.y * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
+            ( pt.x * dpiX ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
+            ( pt.y * dpiY ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
             );
-        QString file;
-        if ( !pt.file.isEmpty() )
-        {
-            file = pt.file;
-            int dotpos = file.lastIndexOf( QLatin1Char( '.' ) );
-            QString ext;
-            if ( dotpos == -1 )
-                file += QString::fromLatin1( ".tex" );
-        }
-        else
-        {
-            file = coreName + QLatin1String( ".tex" );
-        }
+        QString file = pt.file;
         Okular::SourceReference * sourceRef = new Okular::SourceReference( file, pt.row, pt.column );
         refRects[ pt.page ].append( new Okular::SourceRefObjectRect( p, sourceRef ) );
     }
@@ -1556,6 +1466,94 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
             pagesVector[i]->setSourceReferences( refRects.at(i) );
 }
 
+void PDFGenerator::initSynctexParser( const QString& filePath )
+{
+    synctex_scanner = synctex_scanner_new_with_output_file( QFile::encodeName( filePath ), 0, 1);
+}
+
+const Okular::SourceReference * PDFGenerator::dynamicSourceReference( int pageNr, double absX, double absY )
+{
+    if  ( !synctex_scanner )
+        return 0;
+
+    if (synctex_edit_query(synctex_scanner, pageNr + 1, absX * 72. / dpiX, absY * 72. / dpiY) > 0)
+    {
+        synctex_node_t node;
+        // TODO what should we do if there is really more than one node?
+        while (( node = synctex_next_result( synctex_scanner ) ))
+        {
+            int line = synctex_node_line(node);
+            int col = synctex_node_column(node);
+            // column extraction does not seem to be implemented in synctex so far. set the SourceReference default value.
+            if ( col == -1 )
+            {
+                col = 0;
+            }
+            const char *name = synctex_scanner_get_name( synctex_scanner, synctex_node_tag( node ) );
+
+            Okular::SourceReference * sourceRef = new Okular::SourceReference( name, line, col );
+            return sourceRef;
+        }
+    }
+    return 0;
+}
+
+void PDFGenerator::fillViewportFromSourceReference( Okular::DocumentViewport & viewport, const QString & reference ) const
+{
+    if ( !synctex_scanner )
+        return;
+
+    // The reference is of form "src:1111Filename", where "1111"
+    // points to line number 1111 in the file "Filename".
+    // Extract the file name and the numeral part from the reference string.
+    // This will fail if Filename starts with a digit.
+    QString name, lineString;
+    // Remove "src:". Presence of substring has been checked before this 
+    // function is called.
+    name = reference.mid( 4 );
+    // split
+    int nameLength = name.length();
+    int i = 0;
+    for( i = 0; i < nameLength; ++i )
+    {
+        if ( !name[i].isDigit() ) break;
+    }
+    lineString = name.left( i );
+    name = name.mid( i );
+    // Remove spaces.
+    name = name.trimmed();
+    lineString = lineString.trimmed();
+    // Convert line to integer. 
+    bool ok;
+    int line = lineString.toInt( &ok );
+    if (!ok) line = -1;
+    
+    // Use column == -1 for now.
+    if( synctex_display_query( synctex_scanner, name.toLatin1(), line, -1 ) > 0 )
+    {
+        synctex_node_t node;
+        // For now use the first hit. Could possibly be made smarter
+        // in case there are multiple hits.
+        while( ( node = synctex_next_result( synctex_scanner ) ) ) 
+        {
+            // TeX pages start at 1.
+            viewport.pageNumber = synctex_node_page( node ) - 1;
+
+            if ( !viewport.isValid() ) return;
+
+            // TeX small points ...
+            double px = (synctex_node_visible_h( node ) * dpiX) / 72.27;
+            double py = (synctex_node_visible_v( node ) * dpiY) / 72.27;
+            viewport.rePos.normalizedX = px / document()->page(viewport.pageNumber)->width();
+            viewport.rePos.normalizedY = ( py + 0.5 ) / document()->page(viewport.pageNumber)->height();
+            viewport.rePos.enabled = true;
+            viewport.rePos.pos = Okular::DocumentViewport::Center;
+           
+            return;
+        }
+    }
+}
+
 QWidget* PDFGenerator::printConfigurationWidget() const
 {
     if ( !pdfOptionsPage )
@@ -1569,18 +1567,15 @@ bool PDFGenerator::supportsOption( SaveOption option ) const
 {
     switch ( option )
     {
-#ifdef HAVE_POPPLER_0_7
         case SaveChanges:
             return true;
-#endif
         default: ;
     }
     return false;
 }
 
-bool PDFGenerator::save( const QString &fileName, SaveOptions options )
+bool PDFGenerator::save( const QString &fileName, SaveOptions options, QString *errorText )
 {
-#ifdef HAVE_POPPLER_0_7
     Poppler::PDFConverter *pdfConv = pdfdoc->pdfConverter();
 
     pdfConv->setOutputFileName( fileName );
@@ -1589,13 +1584,28 @@ bool PDFGenerator::save( const QString &fileName, SaveOptions options )
 
     QMutexLocker locker( userMutex() );
     bool success = pdfConv->convert();
+#ifdef HAVE_POPPLER_0_12_1
+    if (!success)
+    {
+        switch (pdfConv->lastError())
+        {
+            case Poppler::BaseConverter::NotSupportedInputFileError:
+                *errorText = i18n("Saving files with /Encrypt is not supported.");
+            break;
+
+            case Poppler::BaseConverter::NoError:
+            case Poppler::BaseConverter::FileLockedError:
+                // we can't get here
+            break;
+
+            case Poppler::BaseConverter::OpenOutputError:
+                // the default text message is good for this case
+            break;
+        }
+    }
+#endif
     delete pdfConv;
     return success;
-#else
-    Q_UNUSED( fileName )
-    Q_UNUSED( options )
-    return false;
-#endif
 }
 
 void PDFGenerator::threadFinished()
@@ -1641,10 +1651,13 @@ void PDFGenerator::threadFinished()
     delete outImage;
     if ( !outText.isEmpty() )
     {
-        Okular::TextPage *tp = abstractTextPage( outText, request->page()->height(), 
-                                                 request->page()->width(),request->page()->orientation());
+        Poppler::Page *pp = pdfdoc->page( request->page()->number() );
+        const QSizeF s = pp->pageSizeF();
+        Okular::TextPage *tp = abstractTextPage( outText, s.height(), 
+                                                 s.width(),request->page()->orientation());
         request->page()->setTextPage( tp );
         qDeleteAll(outText);
+        delete pp;
         
         // notify the new generation
         signalTextGenerationDone( request->page(), tp );
@@ -1809,10 +1822,9 @@ void PDFPixmapGeneratorThread::run()
 
     // 1. set OutputDev parameters and Generate contents
     Poppler::Page *pp = d->generator->pdfdoc->page( page->number() );
-    const QSizeF &pageSizeF = pp->pageSizeF();
     
-    double fakeDpiX = width * 72.0 / pageSizeF.width(),
-           fakeDpiY = height * 72.0 / pageSizeF.height();
+    double fakeDpiX = width * d->generator->dpiX / pageWidth,
+           fakeDpiY = height * d->generator->dpiY / pageHeight;
 
     // 2. grab data from the OutputDev and store it locally (note takeIMAGE)
 #ifndef NDEBUG
@@ -1831,11 +1843,7 @@ void PDFPixmapGeneratorThread::run()
 
     if ( genTextPage )
     {
-#ifdef HAVE_POPPLER_0_7
         d->m_textList = pp->textList();
-#else
-        d->m_textList = pp->textList((Poppler::Page::Rotation)d->currentRequest->page()->orientation());
-#endif
     }
     delete pp;
     
