@@ -13,7 +13,7 @@
 #include <qwidget.h>
 
 class QAction;
-class SearchLineEdit;
+class SearchLineWidget;
 
 namespace Okular {
 class Document;
@@ -32,6 +32,7 @@ class FindBar
         Qt::CaseSensitivity caseSensitivity() const;
 
         void focusAndSetCursor();
+        bool maybeHide();
 
     public slots:
         void findNext();
@@ -40,11 +41,13 @@ class FindBar
     private slots:
         void caseSensitivityChanged();
         void fromCurrentPageChanged();
+        void closeAndStopSearch();
 
     private:
-        SearchLineEdit * m_text;
+        SearchLineWidget * m_search;
         QAction * m_caseSensitiveAct;
         QAction * m_fromCurrentPageAct;
+        bool m_active;
 };
 
 
