@@ -20,6 +20,7 @@
 #include <qtimer.h>
 #include <qimage.h>
 #include <qlabel.h>
+#include <qlayout.h>
 #include <qpainter.h>
 #include <qstyle.h>
 #include <qstyleoption.h>
@@ -138,6 +139,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     QString caption = doc->metaData( "DocumentTitle" ).toString();
     if ( caption.trimmed().isEmpty() )
         caption = doc->currentDocument().fileName();
+    caption = i18nc( "[document title/filename] – Presentation", "%1 – Presentation", caption );
     setWindowTitle( KDialog::makeStandardCaption( caption ) );
 
     m_width = -1;
@@ -148,6 +150,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     m_topBar->setObjectName( "presentationBar" );
     m_topBar->setIconSize( QSize( 32, 32 ) );
     m_topBar->setMovable( false );
+    m_topBar->layout()->setMargin(0);
     m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "go-next" : "go-previous" ), i18n( "Previous Page" ), this, SLOT( slotPrevPage() ) );
     m_pagesEdit = new KLineEdit( m_topBar );
     QSizePolicy sp = m_pagesEdit->sizePolicy();
@@ -194,7 +197,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     m_topBar->addWidget( spacer );
     m_topBar->addAction( KIcon( "application-exit" ), i18n( "Exit Presentation Mode" ), this, SLOT( close() ) );
     m_topBar->setAutoFillBackground( true );
-    m_topBar->hide();
+    showTopBar( false );
     // change topbar background color
     QPalette p = m_topBar->palette();
     p.setColor( QPalette::Active, QPalette::Button, Qt::gray );
@@ -233,6 +236,9 @@ PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc
     show();
 
     QTimer::singleShot( 0, this, SLOT( slotDelayedEvents() ) );
+
+    // setFocus() so KCursor::setAutoHideCursor() goes into effect if it's enabled
+    setFocus( Qt::OtherFocusReason );
 }
 
 PresentationWidget::~PresentationWidget()
@@ -425,7 +431,7 @@ void PresentationWidget::keyPressEvent( QKeyEvent * e )
             break;
         case Qt::Key_Escape:
             if ( !m_topBar->isHidden() )
-                m_topBar->hide();
+                showTopBar( false );
             else
                 close();
             break;
@@ -539,7 +545,7 @@ void PresentationWidget::mouseMoveEvent( QMouseEvent * e )
         // hide a shown bar when exiting the area
         if ( e->y() > ( m_topBar->height() + 1 ) )
         {
-            m_topBar->hide();
+            showTopBar( false );
             setFocus( Qt::OtherFocusReason );
         }
     }
@@ -557,7 +563,7 @@ void PresentationWidget::mouseMoveEvent( QMouseEvent * e )
         {
             // show the bar if reaching top 2 pixels
             if ( e->y() <= 1 )
-                m_topBar->show();
+                showTopBar( true );
             // handle "dragging the wheel" if clicking on its geometry
             else if ( ( QApplication::mouseButtons() & Qt::LeftButton ) && m_overlayGeometry.contains( e->pos() ) )
                 overlayClick( e->pos() );
@@ -655,8 +661,17 @@ void PresentationWidget::resizeEvent( QResizeEvent *re )
 
     applyNewScreenSize( re->oldSize() );
 }
-// </widget events>
 
+void PresentationWidget::leaveEvent( QEvent * e )
+{
+    Q_UNUSED( e )
+
+    if ( !m_topBar->isHidden() )
+    {
+        showTopBar( false );
+    }
+}
+// </widget events>
 
 const Okular::Action * PresentationWidget::getLink( int x, int y, QRect * geometry ) const
 {
@@ -834,7 +849,7 @@ void PresentationWidget::generatePage( bool disableTransition )
     }
 
     // update cursor + tooltip
-    if ( Okular::Settings::slidesCursor() != Okular::Settings::EnumSlidesCursor::Hidden )
+    if ( !m_drawingEngine && Okular::Settings::slidesCursor() != Okular::Settings::EnumSlidesCursor::Hidden )
     {
         QPoint p = mapFromGlobal( QCursor::pos() );
         testCursorOnLink( p.x(), p.y() );
@@ -1401,6 +1416,50 @@ void PresentationWidget::allowScreenSaver()
     }
 }
 
+void PresentationWidget::showTopBar( bool show )
+{
+    if ( show )
+    {
+        m_topBar->show();
+
+        // Don't autohide the mouse cursor if it's over the toolbar
+        if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::HiddenDelay )
+        {
+            KCursor::setAutoHideCursor( this, false );
+        }
+
+        // Always show a cursor when topBar is visible
+        if ( !m_drawingEngine )
+        {
+                setCursor( QCursor( Qt::ArrowCursor ) );
+        }
+    }
+    else
+    {
+        m_topBar->hide();
+
+        // Reenable autohide if need be when leaving the toolbar
+        if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::HiddenDelay )
+        {
+            KCursor::setAutoHideCursor( this, true );
+        }
+
+        // Or hide the cursor again if hidden cursor is enabled
+        else if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::Hidden )
+        {
+            // Dont hide the cursor if drawing mode is on
+            if ( !m_drawingEngine )
+            {
+                setCursor( QCursor( Qt::BlankCursor ) );
+            }
+        }
+    }
+
+    // Make sure mouse tracking isn't off after the KCursor::setAutoHideCursor() calls
+    setMouseTracking( true );
+}
+
+
 void PresentationWidget::slotFind()
 {
     if ( !m_searchBar )
